/*
 * Implies.java
 *
 * Created on February 18, 2002, 10:15 AM
 */

package org.semanticweb.dl;

import java.io.*;


/**
 * Implies axiom (for classes).
 * @author  Michael Sintek
 * @version 
 */
public class Implies extends Assertion {

  Expression _expr1, _expr2;
  
  /** Creates new Implies */
  public Implies (Expression expr1, Expression expr2) {
    _expr1 = expr1;
    _expr2 = expr2;
  }
  
  public void print(Syntax syntax, PrintWriter out) {
    if (syntax.XML()) {
      out.print("<IMPLIESC>");
      out.print("<CONCEPT>");
      _expr1.print(syntax, out);
      out.print("</CONCEPT>");
      out.print("<CONCEPT>");
      _expr2.print(syntax, out);
      out.print("</CONCEPT>");
      out.println("</IMPLIESC>");
    } else if (syntax.TRIPLE()) {
      // triple(c1, daml:subClassOf, c2).
      out.print("triple(");
      _expr1.print(syntax, out);
      out.print(",");
      out.print("resource('" + Symbol.DAML_NAMESPACE + "',subClassOf)");
      out.print(",");
      _expr2.print(syntax, out);
      out.println(").");
    } else if (syntax.DOT()) {
      // we use dir=back to make arrows point from upwards
      out.print("\"");
      _expr2.print(syntax, out);
      out.print("\" -> \"");
      _expr1.print(syntax, out);
      out.println("\" [dir=back]");
    } else if (syntax.DAML()) {
      // <daml:Class about='...'> <daml:subClassOf resource='...'/> </>
      out.print("<daml:Class rdf:about='");
      _expr1.print(syntax, out);
      out.println("'>");
      out.print("  <rdfs:subClassOf rdf:resource='");
      _expr2.print(syntax, out);
      out.println("'/>");
      out.println("</daml:Class>");
    } else {
      out.print("(" + syntax.IMPLIES() + " ");
      _expr1.print(syntax, out);
      out.print(" ");
      _expr2.print(syntax, out);
      out.println(")");
    }
  }
  
  public void toSHIQ(KB kb) {
    kb.add(new Implies(_expr1.toSHIQ(kb, CONCEPT), _expr2.toSHIQ(kb, CONCEPT)));
  }

}
