package org.semanticweb.glp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import org.semanticweb.lp.Literal;
import org.semanticweb.utilities.SyntaxHelper;

/**
 *  Title: Triple Description: RDF Query and Inference Language Copyright:
 *  Copyright (c) 2001 Company: Stanford University
 *
 *@author     Stefan Decker
 *@created    January 17, 2002
 *@version    1.0
 */

public class GLPAtom extends GLogicFormula {

    /**
     *  Description of the Field
     */
    String s;
    /**
     *  Description of the Field
     */
    ArrayList a;
    //of GLPTerm


    /**
     *  Constructor for the GLPAtom object
     *
     *@param  s  Description of the Parameter
     *@param  a  Description of the Parameter
     */
    public GLPAtom(String s, ArrayList a) {
        // System.out.println("new GLPAtom: " + s + a);
        this.s = s;
        this.a = a;
    }


    /**
     *  Description of the Method
     *
     *@param  pol  Description of the Parameter
     *@return      Description of the Return Value
     */

    public Literal toLPLiteral(boolean pol) {
        ArrayList args = new ArrayList(a.size());
        for (Iterator i = a.iterator(); i.hasNext(); ) {
            args.add(((GLPTerm) i.next()).toLPTerm());
        }
        return new Literal(pol, s, args);
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public Literal toLPLiteral() {
        return toLPLiteral(true);
        // default is non-negated
    }



    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public String toString() {
        String result = SyntaxHelper.symbol(s, false);
        if (a.size() > 0) {
            result = result.concat("(");
            for (Iterator i = a.iterator(); i.hasNext(); ) {
                result = result.concat(i.next().toString());
                if (i.hasNext()) {
                    result = result.concat(",");
                }
            }
            result = result.concat(")");
        }
        return result;
    }


    /**
     *  Description of the Method
     *
     *@param  h  Description of the Parameter
     *@return    Description of the Return Value
     */
    public GLogicFormula copyFormula(HashMap h) {
        ArrayList na = new ArrayList(a.size());
        for (Iterator i = a.iterator(); i.hasNext(); ) {
            na.add(((GLPTerm) i.next()).copyTerm(h));
        }
        return new GLPAtom(s, na);
    }


    /**
     *  Description of the Method
     *
     *@param  h  Description of the Parameter
     */
    public void renameVariables(HashMap h) {
        ArrayList na = new ArrayList(a.size());
        for (Iterator i = a.iterator(); i.hasNext(); ) {
            ((GLPTerm) i.next()).renameVariables(h);
        }
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    final GLogicFormula copyFormula() {
        ArrayList na = new ArrayList(a.size());
        for (Iterator i = a.iterator(); i.hasNext(); ) {
            na.add(((GLPTerm) i.next()).copyTerm());
        }
        return new GLPAtom(s, na);
    }


    /**
     *  Description of the Method
     *
     *@param  free   Description of the Parameter
     *@param  bound  Description of the Parameter
     */
    public void freeBoundVariables(HashSet free, HashSet bound) {
        for (Iterator i = a.iterator(); i.hasNext(); ) {
            GLPTerm var = (GLPTerm) i.next();
            var.freeBoundVariables(free, bound);
        }
    }
}
