package org.semanticweb.glp;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import org.semanticweb.lp.Term;
import org.semanticweb.lp.FunctorTerm;
import org.semanticweb.utilities.SyntaxHelper;

/**
 *  Title: Triple Description: RDF Query and Inference Language Copyright:
 *  Copyright (c) 2001 Company: Stanford University
 *
 *@author     Stefan Decker
 *@created    January 17, 2002
 *@version    1.0
 */

public class GLPTermFunction extends GLPTerm {
    String s;
    ArrayList a; //of GLPTerm


    /**
     *  Constructor for the GLPTermFunction object
     *
     *@param  s  Description of the Parameter
     *@param  a  Description of the Parameter
     */
    public GLPTermFunction(String s, ArrayList a) {
        this.s = s;
        this.a = a;
    }


    public void renameVariables(HashMap h){
        ArrayList na = new ArrayList(a.size());
        for (Iterator i=a.iterator();i.hasNext();){
          ((GLPTerm)i.next()).renameVariables(h);
        }
    }

    public Term toLPTerm(){
        ArrayList args = new ArrayList(a.size());
        for (Iterator i=a.iterator();i.hasNext();)
          args.add(((GLPTerm)i.next()).toLPTerm());
        return new FunctorTerm(s,args);
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public GLPTerm copyTerm() {
        ArrayList na = new ArrayList(a.size());
        if ((a != null) && (a.size() > 0)) {
            for (Iterator i=a.iterator();i.hasNext();) {
                na.add(((GLPTerm) i.next()).copyTerm());
            }
        }
        return new GLPTermFunction(s, na);
    }


    /**
     *  Description of the Method
     *
     *@param  h  Description of the Parameter
     *@return    Description of the Return Value
     */
    public GLPTerm copyTerm(HashMap h) {
        ArrayList na = new ArrayList(a.size());
        if ((a != null) && (a.size() > 0)) {
            for (Iterator i=a.iterator();i.hasNext();) {
                na.add(((GLPTerm) i.next()).copyTerm(h));
            }
        }
        return new GLPTermFunction(s, na);
    }


    /**
     *  Description of the Method
     *
     *@param  free   Description of the Parameter
     *@param  bound  Description of the Parameter
     */
    public void freeBoundVariables(HashSet free, HashSet bound) {
        if ((a != null) && (a.size() > 0)) {
            for(Iterator i=a.iterator();i.hasNext();) {
               ((GLPTerm) i.next()).freeBoundVariables(free, bound);
            }
        }
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public String toString() {
        String result = SyntaxHelper.symbol(s, false);
        if ((a != null) && (a.size() > 0)) {
            result = result.concat("(");
            for (Iterator i=a.iterator();i.hasNext();) {
                result = result.concat(i.next().toString());
                if (i.hasNext()) {
                    result = result.concat(",");
                }
            }
            result = result.concat(")");
        }
        return result;
    }
}
