package org.semanticweb.triple.inference;

/*
 *  Name: DBInterface.java
 *  Version: 1.1
 *  Purpose: Interface for accessing a database or a ruleset
 *  EvaluateRule and isEvaluable integrated (jan)
 *  History:
 */
import java.io.*;

/**
 *  Description of the Interface
 *
 *@author     Administrator
 *@created    January 17, 2002
 */
public interface DBInterface {

    // Gibt an, ob die Regel durch die Datenbank evaluierbar ist

    /**
     *  Description of the Method
     *
     *@param  sym   Description of the Parameter
     *@param  fact  Description of the Parameter
     */
    public void AddFact(int sym, GroundAtom fact);


    // Fgt ein einzelnes Faktum  hinzu

    /**
     *  Description of the Method
     *
     *@param  sym   Description of the Parameter
     *@param  fact  Description of the Parameter
     */
    public void DeleteFact(int sym, GroundAtom fact);


    // Dies wird fr Optimierungszwecke gebraucht. Liefert die Menge von Termen,

    // die am i-ten Argument aller Fakten des Prdikatsymbols praedikatsymbol auftreten.

    /**
     *  Description of the Method
     *
     *@param  r  Description of the Parameter
     *@return    Description of the Return Value
     */
    public int EvalRule(Rule r);


    // Lscht ein  Faktum

    /**
     *  Description of the Method
     *
     *@param  praedikatsymbol  Description of the Parameter
     *@param  filterterme      Description of the Parameter
     *@return                  Description of the Return Value
     */
    public Atoms GetFacts(int praedikatsymbol, Atoms filterterme);


    // Das ist die wesentliche Funktion zum Zugriff auf Fakten.

    // Es werden auf Fakten mit dem Prdikatsymbol praedikatsymbol zugegriffen.
    // Der Zugriff wird mit Hilfe einer Menge von Filtertermen filterterme eingeschrnkt.
    // Ein Filterterm ist ein Tupel von Termen, die Variablen enthalten knnen.
    // Z.B. ist f(X,a),"hallo",X ein solcher Filterterm.
    // Das Ergebnis des Aufrufs ist eine Menge von Fakten,
    // die zu diesen Filtertermen "passen" (die mit einem der Filterterme matchen).

    // Z.B. enthalte die entsprechende Tabelle der DB die Zeilen:
    // 5, "hallo",8
    // f(7,a), "hallo",7
    // f(ab,a), "hallo",ab
    // f(7,a), "hallo",h
    // f(ab,a), "huhu",ab

    // Die filterterme bestehen nur aus dem einen Filterterm: f(X,a),"hallo",X.
    // Dann ist das Ergebnis des Aufrufs die folgende Menge von Fakten:
    // f(7,a), "hallo", 7
    // f(ab,a), "hallo", ab

    /**
     *  Description of the Method
     *
     *@param  praedikatsymbol  Description of the Parameter
     *@param  i                Description of the Parameter
     *@return                  Description of the Return Value
     */
    public TermSet GetTerms(int praedikatsymbol, int i);


    // Evaluiert die Regel r naiv und schreibt die Ergebnisse wieder in die Datenbank

    // Die Evaluierung wird dabei direkt von der Datenbank ausgefhrt

    /**
     *  Gets the evaluable attribute of the DBInterface object
     *
     *@param  r  Description of the Parameter
     *@return    The evaluable value
     */
    public boolean isEvaluable(Rule r);
}
